/*
 * PHP Reformatter - A tool to make dirty PHP code readable.
 * Copyright (C) 2007 Joris van Rooij <jorrizza@jrrzz.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "preprocessor.h"
#include "postprocessor.h"
#include "version.h"

int main (int argc, char *argv[]) {
  unsigned int i;
  size_t filesize;
  char *code;
  FILE *fh, *fpostprocess;
  struct codeblock_s *codeblock;
  bool loose = FALSE;
  bool tostdout = FALSE;
  bool debug = FALSE;

  if (argc < 2) {
    fprintf(stderr, "PHP Reformatter. Copyright (C) %s %s.\n\n", YEAR, AUTHOR);
    fprintf(stderr, "This program comes with ABSOLUTELY NO WARRANTY; for details see LICENSE.\nThis is free software, and you are welcome to redistribute it\nunder certain conditions; see LICENSE for details.\n\n");
    fprintf(stderr, "Usage: %s [options] filename.php [filename.php ...]\n", BINNAME);
    fprintf(stderr, "Where [options] can be:\n");
    fprintf(stderr, "  %-10s Enable debug mode.\n", "-d");
    fprintf(stderr, "  %-10s Identical to -d.\n", "--debug");
    fprintf(stderr, "  %-10s Loose mode. Needed for lower case defines and OOP code.\n", "-l");
    fprintf(stderr, "  %-10s Identical to -l.\n", "--loose");
    fprintf(stderr, "  %-10s Write to stdout in stead of replacing the original file.\n", "-s");
    fprintf(stderr, "  %-10s Identical to -s.\n", "--stdout");
    fprintf(stderr, "  %-10s Show version information.\n", "-v");
    fprintf(stderr, "  %-10s Identical to -v.\n", "--version");
    fprintf(stderr, "\n");
    return 1;
  }

  for (i = 1; i < argc; i++) {
    if (strncmp(argv[i], "-", 1) == 0) {
      if (strcmp(argv[i], "-l") == 0 || strcmp(argv[i], "--loose") == 0) {
        loose = TRUE;
        continue;
      }
      if (strcmp(argv[i], "-s") == 0 || strcmp(argv[i], "--stdout") == 0) {
        tostdout = TRUE;
        continue;
      }
      if (strcmp(argv[i], "-d") == 0 || strcmp(argv[i], "--debug") == 0) {
        debug = TRUE;
        continue;
      }
      if (strcmp(argv[i], "-v") == 0 || strcmp(argv[i], "--version") == 0) {
        fprintf(stderr, "PHP Reformatter. Copyright (C) %s %s.\n\n", YEAR, AUTHOR);
        fprintf(stderr, "This program comes with ABSOLUTELY NO WARRANTY; for details see LICENSE.\nThis is free software, and you are welcome to redistribute it\nunder certain conditions; see LICENSE for details.\n\n");
        fprintf(stderr, "Version:\t\t %s.\n", VERSION);
        fprintf(stderr, "Platform:\t\t %s.\n", PLATFORM);
        fprintf(stderr, "Build date:\t\t %s.\n", BUILDDATE);
        return 0;
      }
      fprintf(stderr, "FATAL ERROR: Unknown option '%s'.\n", argv[i]);
      return 1;
    }
    if ((fh = fopen(argv[i], "rb")) != NULL) {
      fseek(fh, 0, SEEK_END);
      filesize = ftell(fh) + 1;
      rewind(fh);
      if ((code = (char *) malloc(filesize + 1)) == NULL) {
        fprintf(stderr, "FATAL ERROR: Cannot alloc %d bytes of memory.\n", (unsigned int) filesize + 1);
        return 1;
      }
      memset(code, 0, filesize + 1);
      fread(code, 1, filesize, fh);

      codeblock_init(&codeblock, PHP_STDOUT_DATA, "");

      /* Preprocessor */
      if (!preprocess(code, filesize, codeblock, argv[i], loose)) {
        fprintf(stderr, "Preprocessor: %s: Error: Cannot preprocess file.\n\n", argv[i]);
        continue;
      }
      fprintf(stderr, "Preprocessor: %s: OK: Preprocessed file.\n\n", argv[i]);

      /* Debugger */
      if (debug)
        codeblock_debug(codeblock, argv[i]);

      /* Postprocessor */
      if (tostdout)
        fpostprocess = stdout;
      else {
        fclose(fh);
        fh = fopen(argv[i], "wb");
        fpostprocess = fh;
      }
      if (!postprocess(fpostprocess, codeblock)) {
        fprintf(stderr, "Postprocessor: %s: Error: Cannot postprocess file.\n\n", argv[i]);
        continue;
      }
      fprintf(stderr, "Postprocessor: %s: OK: Postprocessed file.\n\n", argv[i]);

      codeblock_free(codeblock);

      free(code);
      fclose(fh);
    } else
      fprintf(stderr, "%s: %s.\n", argv[i], strerror(errno));
  }

  return 0;
}
