/*
 * PHP Reformatter - A tool to make dirty PHP code readable.
 * Copyright (C) 2007 Joris van Rooij <jorrizza@jrrzz.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <stdio.h>
#include <string.h>
#include "postprocessor.h"
#include "bool.h"
#include "standards.h"

unsigned int indentcnt = 0;

void indent (void) {
  indentcnt += INDENTCNT;
}

void unindent (void) {
  if (indentcnt >= INDENTCNT)
    indentcnt -= INDENTCNT;
}

void findent (FILE *fh) {
  unsigned int i;

  for (i = 0; i < indentcnt; i++)
    fputc(INDENTCHR, fh);
}

void print_indented_string (FILE *fh, char *string) {
  bool is_stdout = (strncmp(string, "<<<", 3) == 0);

  while (*string != 0x00) {
    if (*string == 0x0A && !KEEP_STRING_INDENTATION && !is_stdout) {
      string++;
      while (*string == 0x20 || *string == 0x09)
        string++;
      NEWLINE(fh);
    } else {
      fputc(*string, fh);
      string++;
    }
  }
}

void print_indented_comment (FILE *fh, char *string) {
  if (strncmp(string, "/**", strlen("/**")) == 0)
    string += strlen("/**");
  else if (strncmp(string, "/*", strlen("/*")) == 0)
    string += strlen("/*");

  NEWLINE(fh);
  fputs("/**", fh);
  NEWLINE(fh);
  fputs(" * ", fh);

  while (*string != 0x00 && strncmp(string, "*/", strlen("*/")) != 0) {
    if (*string == 0x0A) {
      string++;
      NEWLINE(fh);
      fputs(" * ", fh);
      while (*string == 0x20 || *string == 0x09)
        string++;
      if (strncmp(string, "*/", strlen("*/")) == 0)
        continue;
      while ((*string == 0x20 || *string == 0x09 || *string == 0x0A || *string == 0x2A) && strncmp(string, "*/", strlen("*/")) != 0)
        string++;
    } else {
      fputc(*string, fh);
      string++;
    }
  }

  NEWLINE(fh);
  fputs(" */", fh);
  NEWLINE(fh);
}

int postprocess (FILE *fh, struct codeblock_s *codeblock) {
  bool php_in_loop_do = FALSE;
  bool php_in_loop_single = FALSE;
  bool php_in_loop_short = FALSE;
  bool php_in_if_short = FALSE;
  bool php_in_array_multiline = FALSE;
  indentcnt = 0;

  if (fh == NULL)
    return 0;

  while (codeblock != NULL) {
    switch (codeblock->tag) {
      case PHP_START_TAG:
        fprintf(fh, "%s", codeblock->content);
        NEWLINE(fh);
        break;
      case PHP_END_TAG:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_COMMENT_SINGLE:
        fprintf(fh, "%s", codeblock->content);
        findent(fh);
        break;
      case PHP_COMMENT_MULTI:
        NEWLINE(fh);
        print_indented_comment(fh, codeblock->content);
        break;
      case PHP_TYPECAST_INT:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_TYPECAST_FLOAT:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_TYPECAST_STRING:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_TYPECAST_ARRAY:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_TYPECAST_OBJECT:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_FUNCTION_TYPE:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_FUNCTION_NAME:
        fprintf(fh, "%s%s", codeblock->content, (SPLIT_FUNCTION_DECLARATION ? " " : ""));
        break;
      case PHP_FUNCTION_BODY_START:
        fprintf(fh, "%s%s", codeblock->content, (PAD_BLOCK_INNER_BODY ? " " : ""));
        break;
      case PHP_FUNCTION_BODY_END:
        fprintf(fh, "%s%s%s", (PAD_BLOCK_INNER_BODY ? " " : ""), codeblock->content, (PAD_BLOCK_OUTER_BODY ? " " : ""));
        if (NEWLINE_AFTER_CLAUSE) {
          NEWLINE(fh);
        }
        break;
      case PHP_FUNCTION_BODY_SEPERATOR:
        fprintf(fh, "%s%s", codeblock->content, (PAD_FUNCTION_ARGUMENTS ? " " : ""));
        break;
      case PHP_FUNCTION_START:
        fprintf(fh, "%s", codeblock->content);
        indent();
        NEWLINE(fh);
        break;
      case PHP_FUNCTION_END:
        unindent();
        NEWLINE(fh);
        fprintf(fh, "%s", codeblock->content);
        if (codeblock->next != NULL) {
          if (codeblock->next->tag != PHP_ELSEIF_TYPE && codeblock->next->tag != PHP_ELSE_TYPE && codeblock->next->tag != PHP_IF_END && codeblock->next->tag != PHP_FUNCTION_END && codeblock->next->tag != PHP_LOOP_END && codeblock->next->tag != PHP_SWITCH_END && codeblock->next->tag != PHP_OOP_CLASS_END) {
            NEWLINE(fh);
          }
        }
        if (NEWLINE_AFTER_FUNCTION) {
          NEWLINE(fh);
        }
        break;
      case PHP_LOOP_WHILE:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_LOOP_FOREACH:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_LOOP_FOR:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_LOOP_DO:
        fprintf(fh, "%s%s", codeblock->content, (PAD_BLOCK_OUTER_BODY ? " " : ""));
        php_in_loop_do = TRUE;
        break;
      case PHP_LOOP_BODY_START:
        fprintf(fh, "%s%s%s", (PAD_BLOCK_OUTER_BODY ? " " : ""), codeblock->content, (PAD_BLOCK_INNER_BODY ? " " : ""));
        break;
      case PHP_LOOP_BODY_END:
        fprintf(fh, "%s%s", (PAD_BLOCK_INNER_BODY ? " " : ""), codeblock->content);
        if (codeblock->next != NULL) {
          if (codeblock->next->tag != PHP_LINE_TERMINATOR && PAD_BLOCK_OUTER_BODY)
            fprintf(fh, " ");
          else if (codeblock->next->tag == PHP_LINE_TERMINATOR)
            php_in_loop_short = TRUE;
          if (codeblock->next->tag != PHP_LOOP_START && codeblock->next->tag != PHP_LINE_TERMINATOR) {
            indent();
            NEWLINE(fh);
            php_in_loop_single = TRUE;
          } else if (NEWLINE_AFTER_CLAUSE && codeblock->next->tag != PHP_LINE_TERMINATOR) {
            NEWLINE(fh);
          }
        }
        break;
      case PHP_LOOP_BODY_SEPERATOR:
        fprintf(fh, "%s%s", codeblock->content, (PAD_LOOP_ARGUMENTS ? " " : ""));
        break;
      case PHP_LOOP_START:
        fprintf(fh, "%s", codeblock->content);
        indent();
        NEWLINE(fh);
        break;
      case PHP_LOOP_END:
        unindent();
        NEWLINE(fh);
        fprintf(fh, "%s", codeblock->content);
        if (NEWLINE_AFTER_LOOP && !php_in_loop_do) {
          NEWLINE(fh);
        }
        if (!php_in_loop_do) {
          if (codeblock->next != NULL) {
            if (codeblock->next->tag != PHP_IF_END && codeblock->next->tag != PHP_FUNCTION_END && codeblock->next->tag != PHP_LOOP_END && codeblock->next->tag != PHP_SWITCH_END && codeblock->next->tag != PHP_OOP_CLASS_END) {
              NEWLINE(fh);
            }
          }
        } else {
          fprintf(fh, "%s", (PAD_BLOCK_OUTER_BODY ? " " : ""));
          if (NEWLINE_AFTER_CLAUSE) {
            NEWLINE(fh);
          }
          php_in_loop_do = FALSE;
        }
        break;
      case PHP_SWITCH_TYPE:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_SWITCH_BODY_START:
        fprintf(fh, "%s%s%s", (PAD_BLOCK_OUTER_BODY ? " " : ""), codeblock->content, (PAD_BLOCK_INNER_BODY ? " " : ""));
        break;
      case PHP_SWITCH_BODY_END:
        fprintf(fh, "%s%s%s", (PAD_BLOCK_INNER_BODY ? " " : ""), codeblock->content, (PAD_BLOCK_OUTER_BODY ? " " : ""));
        if (NEWLINE_AFTER_CLAUSE) {
          NEWLINE(fh);
        }
        break;
      case PHP_SWITCH_START:
        fprintf(fh, "%s", codeblock->content);
        indent();
        NEWLINE(fh);
        break;
      case PHP_SWITCH_END:
        unindent();
        if (INDENT_AFTER_CASE)
          unindent();
        NEWLINE(fh);
        fprintf(fh, "%s", codeblock->content);
        if (codeblock->next != NULL) {
          if (codeblock->next->tag != PHP_IF_END && codeblock->next->tag != PHP_FUNCTION_END && codeblock->next->tag != PHP_LOOP_END && codeblock->next->tag != PHP_SWITCH_END && codeblock->next->tag != PHP_OOP_CLASS_END) {
            NEWLINE(fh);
          }
        }
        if (NEWLINE_AFTER_FUNCTION) {
          NEWLINE(fh);
        }
        break;
      case PHP_SWITCH_CASE:
        fprintf(fh, "%s", codeblock->content);
        if (codeblock->next != NULL) {
          if (INDENT_AFTER_CASE && codeblock->next->tag != PHP_KEYWORD_CASE && codeblock->next->tag != PHP_KEYWORD_DEFAULT && codeblock->next->tag != PHP_COMMENT_SINGLE && codeblock->next->tag != PHP_COMMENT_MULTI)
            indent();
        }
        NEWLINE(fh);
        break;
      case PHP_OOP_CLASS_TYPE:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_OOP_CLASS_NAME:
        fprintf(fh, "%s%s", codeblock->content, (PAD_BLOCK_OUTER_BODY ? " " : ""));
        break;
      case PHP_OOP_CLASS_START:
        if (NEWLINE_AFTER_CLAUSE) {
          NEWLINE(fh);
        }
        fprintf(fh, "%s", codeblock->content);
        indent();
        NEWLINE(fh);
        break;
      case PHP_OOP_CLASS_END:
        unindent();
        NEWLINE(fh);
        fprintf(fh, "%s", codeblock->content);
        if (codeblock->next != NULL) {
          if (codeblock->next->tag != PHP_IF_END && codeblock->next->tag != PHP_FUNCTION_END && codeblock->next->tag != PHP_LOOP_END && codeblock->next->tag != PHP_SWITCH_END && codeblock->next->tag != PHP_OOP_CLASS_END) {
            NEWLINE(fh);
          }
        }
        if (NEWLINE_AFTER_FUNCTION) {
          NEWLINE(fh);
        }
        break;
      case PHP_OOP_ARROW:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_OOP_SCOPE_RESOLUTION:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_ERROR_SILENCE:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_INCREMENTOR:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_DECREMENTOR:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_ARITHMETIC_NEGATION:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_ARITHMETIC_NOT:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_REFERENCE:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_ARRAY_ARROW:
        fprintf(fh, "%s%s%s", (PAD_COMPARISON ? " " : ""), codeblock->content, (PAD_COMPARISON ? " " : ""));
        break;
      case PHP_STRING:
        print_indented_string(fh, codeblock->content);
        break;
      case PHP_ARITHMETIC:
        fprintf(fh, "%s%s%s", (PAD_ARITHMETIC ? " " : ""), codeblock->content, (PAD_ARITHMETIC ? " " : ""));
        break;
      case PHP_COMPARISON:
        fprintf(fh, "%s%s%s", (PAD_COMPARISON ? " " : ""), codeblock->content, (PAD_COMPARISON ? " " : ""));
        break;
      case PHP_ASSIGNMENT:
        fprintf(fh, "%s%s%s", (PAD_ASSIGNMENT ? " " : ""), codeblock->content, (PAD_ASSIGNMENT ? " " : ""));
        break;
      case PHP_ARITHMETIC_INV:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_VARIABLE:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_NUMBER:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_ARRAY_TYPE:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_ARRAY_START:
        fprintf(fh, "%s", codeblock->content);
        if (codeblock->next != NULL) {
          if (ARRAY_MULTILINE && codeblock->next->tag != PHP_ARRAY_END) {
            indent();
            NEWLINE(fh);
            php_in_array_multiline = TRUE;
          }
        }
        break;
      case PHP_ARRAY_END:
        if (ARRAY_MULTILINE && php_in_array_multiline) {
          unindent();
          NEWLINE(fh);
          php_in_array_multiline = FALSE;
        }
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_ARRAY_SEPERATOR:
        fprintf(fh, "%s%s", codeblock->content, (PAD_FUNCTION_ARGUMENTS ? " " : ""));
        if (ARRAY_MULTILINE) {
          NEWLINE(fh);
        }
        break;
      case PHP_IF_TYPE:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_IF_BODY_START:
        fprintf(fh, "%s%s%s", (PAD_BLOCK_OUTER_BODY ? " " : ""), codeblock->content, (PAD_BLOCK_INNER_BODY ? " " : ""));
        break;
      case PHP_IF_BODY_END:
        fprintf(fh, "%s%s%s", (PAD_BLOCK_INNER_BODY ? " " : ""), codeblock->content, (PAD_BLOCK_OUTER_BODY ? " " : ""));
        if (codeblock->next != NULL) {
          if (codeblock->next->tag != PHP_IF_START) {
            indent();
            NEWLINE(fh);
            php_in_if_short = TRUE;
          } else if (NEWLINE_AFTER_CLAUSE) {
            NEWLINE(fh);
          }
        }
        break;
      case PHP_IF_START:
        fprintf(fh, "%s", codeblock->content);
        indent();
        NEWLINE(fh);
        break;
      case PHP_IF_END:
        unindent();
        NEWLINE(fh);
        fprintf(fh, "%s", codeblock->content);
        if (codeblock->next != NULL) {
          if (NEWLINE_AFTER_CLAUSE && codeblock->next->tag != PHP_IF_END && codeblock->next->tag != PHP_FUNCTION_END && codeblock->next->tag != PHP_LOOP_END && codeblock->next->tag != PHP_SWITCH_END && codeblock->next->tag != PHP_OOP_CLASS_END) {
            NEWLINE(fh);
          }
          if (NEWLINE_AFTER_IFBLOCK && codeblock->next->tag != PHP_ELSEIF_TYPE && codeblock->next->tag != PHP_ELSE_TYPE) {
            NEWLINE(fh);
          }
          if (!NEWLINE_AFTER_CLAUSE && (codeblock->next->tag == PHP_ELSEIF_TYPE || codeblock->next->tag == PHP_ELSE_TYPE))
            fprintf(fh, "%s", (PAD_BLOCK_OUTER_BODY ? " " : ""));
          if (!NEWLINE_AFTER_CLAUSE && codeblock->next->tag != PHP_ELSEIF_TYPE && codeblock->next->tag != PHP_ELSE_TYPE && codeblock->next->tag != PHP_IF_END && codeblock->next->tag != PHP_FUNCTION_END && codeblock->next->tag != PHP_LOOP_END && codeblock->next->tag != PHP_SWITCH_END && codeblock->next->tag != PHP_OOP_CLASS_END) {
            NEWLINE(fh);
          }
        }
        break;
      case PHP_ELSEIF_TYPE:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_ELSE_TYPE:
        fprintf(fh, "%s", codeblock->content);
        if (codeblock->next != NULL) {
          if (codeblock->next->tag != PHP_IF_START && codeblock->next->tag != PHP_IF_TYPE) {
            indent();
            NEWLINE(fh);
            php_in_if_short = TRUE;
          } else if (NEWLINE_AFTER_CLAUSE) {
            NEWLINE(fh);
          } else
            fprintf(fh, "%s", (PAD_BLOCK_OUTER_BODY ? " " : ""));
        }
        break;
      case PHP_KEY_START:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_KEY_END:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_SUBSTRING_START:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_SUBSTRING_END:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_CONCATENATOR:
        fprintf(fh, "%s%s%s", (PAD_CONCATENATOR ? " " : ""), codeblock->content, (PAD_CONCATENATOR ? " " : ""));
        break;
      case PHP_LINE_TERMINATOR:
        fprintf(fh, "%s", codeblock->content);
        if (php_in_loop_single) {
          unindent();
          php_in_loop_single = FALSE;
          if (NEWLINE_AFTER_LOOP) {
            NEWLINE(fh);
          }
        }
        if (php_in_loop_short) {
          php_in_loop_short = FALSE;
          if (NEWLINE_AFTER_LOOP) {
            NEWLINE(fh);
          }
        }
        if (codeblock->next != NULL) {
          if (INDENT_AFTER_CASE && (codeblock->next->tag == PHP_KEYWORD_CASE || codeblock->next->tag == PHP_KEYWORD_DEFAULT))
            unindent();
          if (php_in_if_short && (codeblock->next->tag == PHP_ELSEIF_TYPE || codeblock->next->tag == PHP_ELSE_TYPE)) {
            php_in_if_short = FALSE;
            unindent();
            NEWLINE(fh);
          } else if (php_in_if_short) {
            php_in_if_short = FALSE;
            unindent();
            if (NEWLINE_AFTER_IFBLOCK) {
              NEWLINE(fh);
            }
          }
          if (NEWLINE_AFTER_LINE_TERMINATOR && codeblock->next->tag != PHP_FUNCTION_END && codeblock->next->tag != PHP_LOOP_END && codeblock->next->tag != PHP_SWITCH_END && codeblock->next->tag != PHP_ELSEIF_TYPE && codeblock->next->tag != PHP_ELSE_TYPE && codeblock->next->tag != PHP_IF_END && codeblock->next->tag != PHP_OOP_CLASS_END) {
            NEWLINE(fh);
          }
        }
        break;
      case PHP_FUNCTION_CALL:
        fprintf(fh, "%s%s", codeblock->content, (SPLIT_FUNCTION_CALL ? " " : ""));
        break;
      case PHP_FUNCTION_CALL_BODY_START:
        fprintf(fh, "%s%s", codeblock->content, (PAD_BLOCK_INNER_BODY ? " " : ""));
        break;
      case PHP_FUNCTION_CALL_BODY_END:
        fprintf(fh, "%s%s", (PAD_BLOCK_INNER_BODY ? " " : ""), codeblock->content);
        break;
      case PHP_FUNCTION_CALL_BODY_SEPERATOR:
        fprintf(fh, "%s%s", codeblock->content, (PAD_FUNCTION_ARGUMENTS ? " " : ""));
        break;
      case PHP_BOOLGROUP_START:
        fprintf(fh, "%s%s", codeblock->content, (PAD_BLOCK_INNER_BODY ? " " : ""));
        break;
      case PHP_BOOLGROUP_END:
        fprintf(fh, "%s%s", (PAD_BLOCK_INNER_BODY ? " " : ""), codeblock->content);
        break;
      case PHP_KEYWORD_BREAK:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_KEYWORD_EXIT:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_KEYWORD_DIE:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_KEYWORD_ECHO:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_KEYWORD_PRINT:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_KEYWORD_REQUIRE:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_KEYWORD_REQUIREONCE:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_KEYWORD_INCLUDE:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_KEYWORD_INCLUDEONCE:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_KEYWORD_RETURN:
        fprintf(fh, "%s", codeblock->content);
        if (codeblock->next != NULL) {
          if (codeblock->next->tag != PHP_LINE_TERMINATOR)
            fprintf(fh, " ");
        }
        break;
      case PHP_KEYWORD_CONTINUE:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_KEYWORD_AS:
        fprintf(fh, " %s ", codeblock->content);
        break;
      case PHP_KEYWORD_GLOBAL:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_KEYWORD_TRUE:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_KEYWORD_FALSE:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_KEYWORD_CASE:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_KEYWORD_DEFAULT:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_OOP_KEYWORD_NEW:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_OOP_KEYWORD_EXTENDS:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_OOP_KEYWORD_PUBLIC:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_OOP_KEYWORD_PROTECTED:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_OOP_KEYWORD_PRIVATE:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_OOP_KEYWORD_VAR:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_OOP_KEYWORD_CONST:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_OOP_KEYWORD_STATIC:
        fprintf(fh, "%s ", codeblock->content);
        break;
      case PHP_SHORTHAND_IF:
        fprintf(fh, "%s%s%s", (PAD_SHORTHAND ? " " : ""), codeblock->content, (PAD_SHORTHAND ? " " : ""));
        break;
      case PHP_SHORTHAND_ELSE:
        fprintf(fh, "%s%s%s", (PAD_SHORTHAND ? " " : ""), codeblock->content, (PAD_SHORTHAND ? " " : ""));
        break;
      case PHP_DEFINED:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_UNDEFINED:
        fprintf(fh, "%s", codeblock->content);
        break;
      case PHP_MISC_SEPERATOR:
        fprintf(fh, "%s%s", codeblock->content, (PAD_FUNCTION_ARGUMENTS ? " " : ""));
        break;
      case PHP_STDOUT_DATA:
        fprintf(fh, "%s", codeblock->content);
        break;
    }
    codeblock = codeblock->next;
  }

  return 1;
}
